@@ -3,7 +3,7 @@
 
  Part of the Routino routing software.
  ******************/ /******************
- This file Copyright 2008-2011 Andrew M. Bishop
+ This file Copyright 2008-2015, 2019 Andrew M. Bishop
 
  This program is free software: you can redistribute it and/or modify
  it under the terms of the GNU Affero General Public License as published by
@@ -30,6 +30,7 @@
 
 #include "typesx.h"
 
+#include "cache.h"
 #include "files.h"
 
 
@@ -44,18 +45,21 @@ struct _NodeX
  latlong_t    latitude;         /*+ The node latitude. +*/
  latlong_t    longitude;        /*+ The node longitude. +*/
 
- transports_t allow;            /*+ The node allowed traffic. +*/
+ transports_t allow;            /*+ The types of transport that are allowed through the node. +*/
 
- uint16_t     flags;            /*+ The node flags. +*/
+ nodeflags_t  flags;            /*+ Flags containing extra information (e.g. super-node, turn restriction). +*/
 };
 
 /*+ A structure containing a set of nodes (memory format). +*/
 struct _NodesX
 {
- char     *filename;            /*+ The name of the temporary file. +*/
- int       fd;                  /*+ The file descriptor of the temporary file. +*/
+ char     *filename;            /*+ The name of the intermediate file (for the NodesX). +*/
+ char     *filename_tmp;        /*+ The name of the temporary file (for the NodesX). +*/
+
+ int       fd;                  /*+ The file descriptor of the open file (for the NodesX). +*/
 
  index_t   number;              /*+ The number of extended nodes still being considered. +*/
+ index_t   knumber;             /*+ The number of extended nodes kept for next time. +*/
 
 #if !SLIM
 
@@ -63,15 +67,20 @@ struct _NodesX
 
 #else
 
- NodeX     cached[2];           /*+ Two cached nodes read from the file in slim mode. +*/
+ NodeX     cached[3];           /*+ Three cached extended nodes read from the file in slim mode. +*/
+ index_t   incache[3];          /*+ The indexes of the cached extended nodes. +*/
+
+ NodeXCache *cache;             /*+ A RAM cache of extended nodes read from the file. +*/
 
 #endif
 
  node_t   *idata;               /*+ The extended node IDs (sorted by ID). +*/
 
+ index_t  *pdata;               /*+ The node indexes after pruning. +*/
+
  index_t  *gdata;               /*+ The final node indexes (sorted geographically). +*/
 
- uint8_t  *super;               /*+ A bit-mask marker for super nodes (same order as sorted nodes). +*/
+ BitMask  *super;               /*+ A bit-mask marker for super nodes (same order as sorted nodes). +*/
 
  index_t   latbins;             /*+ The number of bins containing latitude. +*/
  index_t   lonbins;             /*+ The number of bins containing longitude. +*/
@@ -83,22 +92,23 @@ struct _NodesX
 
 /* Functions in nodesx.c */
 
-NodesX *NewNodeList(int append);
+NodesX *NewNodeList(int append,int readonly);
 void FreeNodeList(NodesX *nodesx,int keep);
 
-void SaveNodeList(NodesX *nodesx,const char *filename);
+void AppendNodeList(NodesX *nodesx,node_t id,double latitude,double longitude,transports_t allow,nodeflags_t flags);
+void FinishNodeList(NodesX *nodesx);
 
 index_t IndexNodeX(NodesX *nodesx,node_t id);
 
-void AppendNode(NodesX *nodesx,node_t id,double latitude,double longitude,transports_t allow,uint16_t flags);
-
 void SortNodeList(NodesX *nodesx);
 
+void RemoveNonHighwayNodes(NodesX *nodesx,WaysX *waysx,int keep);
+
+void RemovePrunedNodes(NodesX *nodesx,SegmentsX *segmentsx);
+
 void SortNodeListGeographically(NodesX *nodesx);
 
-void RemoveNonHighwayNodes(NodesX *nodesx,SegmentsX *segmentsx);
-
-void UpdateNodes(NodesX *nodesx,SegmentsX *segmentsx);
+void SaveNodeList(NodesX *nodesx,const char *filename,SegmentsX *segmentsx);
 
 
 /* Macros and inline functions */
@@ -107,13 +117,33 @@ void UpdateNodes(NodesX *nodesx,SegmentsX *segmentsx);
 
 #define LookupNodeX(nodesx,index,position)      &(nodesx)->data[index]
   
-#define PutBackNodeX(nodesx,index,position)     /* nop */
+#define PutBackNodeX(nodesx,nodex)              while(0) { /* nop */ }
 
 #else
 
-static NodeX *LookupNodeX(NodesX *nodesx,index_t index,int position);
+/* Prototypes */
 
-static void PutBackNodeX(NodesX *nodesx,index_t index,int position);
+static inline NodeX *LookupNodeX(NodesX *nodesx,index_t index,int position);
+
+static inline void PutBackNodeX(NodesX *nodesx,NodeX *nodex);
+
+CACHE_NEWCACHE_PROTO(NodeX)
+CACHE_DELETECACHE_PROTO(NodeX)
+CACHE_FETCHCACHE_PROTO(NodeX)
+CACHE_REPLACECACHE_PROTO(NodeX)
+CACHE_INVALIDATECACHE_PROTO(NodeX)
+
+/* Data type */
+
+CACHE_STRUCTURE(NodeX)
+
+/* Inline functions */
+
+CACHE_NEWCACHE(NodeX)
+CACHE_DELETECACHE(NodeX)
+CACHE_FETCHCACHE(NodeX)
+CACHE_REPLACECACHE(NodeX)
+CACHE_INVALIDATECACHE(NodeX)
 
 
 /*++++++++++++++++++++++++++++++++++++++
@@ -130,7 +160,9 @@ static void PutBackNodeX(NodesX *nodesx,index_t index,int position);
 
 static inline NodeX *LookupNodeX(NodesX *nodesx,index_t index,int position)
 {
- SeekReadFile(nodesx->fd,&nodesx->cached[position-1],sizeof(NodeX),(off_t)index*sizeof(NodeX));
+ nodesx->cached[position-1]=*FetchCachedNodeX(nodesx->cache,index,nodesx->fd,0);
+
+ nodesx->incache[position-1]=index;
 
  return(&nodesx->cached[position-1]);
 }
@@ -141,14 +173,14 @@ static inline NodeX *LookupNodeX(NodesX *nodesx,index_t index,int position)
 
   NodesX *nodesx The set of nodes to modify.
 
-  index_t index The node index to put back.
-
-  int position The position in the cache to use.
+  NodeX *nodex The extended node to be put back.
   ++++++++++++++++++++++++++++++++++++++*/
 
-static inline void PutBackNodeX(NodesX *nodesx,index_t index,int position)
+static inline void PutBackNodeX(NodesX *nodesx,NodeX *nodex)
 {
- SeekWriteFile(nodesx->fd,&nodesx->cached[position-1],sizeof(NodeX),(off_t)index*sizeof(NodeX));
+ int position1=nodex-&nodesx->cached[0];
+
+ ReplaceCachedNodeX(nodesx->cache,nodex,nodesx->incache[position1],nodesx->fd,0);
 }
 
 #endif /* SLIM */
